Scriptname RRAttributesScript extends Quest  
{Realistic Regeneration : Attributes}

;
; GLOBAL VARIABLES --------------------------------------------------------------------------------
;

; Game
Actor Property PlayerREF Auto


GlobalVariable Property RRAttrHealthModifier Auto
GlobalVariable Property RRAttrMagickaModifier Auto
GlobalVariable Property RRAttrStaminaModifier Auto
GlobalVariable Property RRAttrCarryWeightModifier Auto


; Scripts
RRControlScript Property RRControl Auto

; Timers
GlobalVariable Property RRUpdateAttributesAt Auto

; Hide In UI
GlobalVariable Property RRHideInUI Auto


;
; SPELLS ------------------------------------------------------------------------------------------
;

; legacy
Spell Property RRAttributesSpell Auto

; correct spells: added in 3.08
Spell Property RRAttributeCarryWeightSpell Auto
Spell Property RRAttributeHealthSpell Auto
Spell Property RRAttributeMagickaSpell Auto
Spell Property RRAttributeStaminaSpell Auto


;
; PRIVATE VARIABLES -------------------------------------------------------------------------------
;

int Property HideInUI    = 0x00008000 autoReadOnly
int Property Detrimental = 0x00000004 autoReadOnly
GlobalVariable[] Property gvModifiersList Auto Hidden
Spell[] Property spAttributesList Auto Hidden

;
; FUNCTIONS ---------------------------------------------------------------------------------------
;

Function initializeAll()
	initializeVariables()
EndFunction

Function updateAll()
	initializeVariables()

	; remove legacy spell
	if ( PlayerREF.HasSpell( RRAttributesSpell ) )
		PlayerREF.RemoveSpell( RRAttributesSpell )
	endIf

	; re-apply
	setPlayerSettings()
	addAttributesSpells()
EndFunction

Function reset()
	deactivate()

	if ( RRControl.isActive() )
		activate()
	endIf
EndFunction

Function activate()
	removeAllSpells()
	setPlayerSettings()
	GotoState( "onAddAttributes" )
	RegisterForSingleUpdate( RRUpdateAttributesAt.GetValue() )
EndFunction

Function deactivate()
	removeAllSpells()
	UnregisterForUpdate()
	GotoState( "Inactive" )
EndFunction

Function initializeVariables()
	gvModifiersList = new GlobalVariable[4]
	gvModifiersList[0] = RRAttrHealthModifier
	gvModifiersList[1] = RRAttrMagickaModifier
	gvModifiersList[2] = RRAttrStaminaModifier
	gvModifiersList[3] = RRAttrCarryWeightModifier

	spAttributesList = new Spell[4]
	spAttributesList[0] = RRAttributeHealthSpell
	spAttributesList[1] = RRAttributeMagickaSpell
	spAttributesList[2] = RRAttributeStaminaSpell
	spAttributesList[3] = RRAttributeCarryWeightSpell
EndFunction

Function removeAllSpells()
	int iIndex = 0

	While iIndex < gvModifiersList.Length
		Spell aSpell = spAttributesList[iIndex]

		If PlayerREF.HasSpell( aSpell )
			PlayerREF.RemoveSpell( aSpell )
		EndIf

		iIndex += 1
	EndWhile
EndFunction

Function addAttributesSpells()
	int iIndex = 0
	bool bDebug = RRControl.isDebug()

	While iIndex < gvModifiersList.Length
		Spell aSpell = spAttributesList[iIndex]

		If aSpell.GetNthEffectMagnitude( 0 ) != 0.000000
			PlayerREF.AddSpell( aSpell, bDebug )
		EndIf

		iIndex += 1
	EndWhile
EndFunction

Function setPlayerSettings()
	int iIndex = 0
	bool bHide = (RRHideInUI.GetValueInt() == 1)

	While iIndex < gvModifiersList.Length
		Spell aSpell = spAttributesList[iIndex]
		MagicEffect spellEffect = aSpell.GetNthEffectMagicEffect( 0 )

		if bHide
			if ( !spellEffect.IsEffectFlagSet( HideInUI ) )
				spellEffect.SetEffectFlag( HideInUI )
			endIf
		else
			if ( spellEffect.IsEffectFlagSet( HideInUI ) )
				spellEffect.ClearEffectFlag( HideInUI )
			endIf
		endIf

		float fMag = gvModifiersList[iIndex].GetValue()

		if ( fMag >= 0.000000 ) ; increase
			if ( spellEffect.IsEffectFlagSet( Detrimental ) )
				spellEffect.ClearEffectFlag( Detrimental )
			endIf
		else
			if ( !spellEffect.IsEffectFlagSet( Detrimental ) )
				spellEffect.SetEffectFlag( Detrimental )
			endIf
		endif

		aSpell.SetNthEffectMagnitude( 0, Math.abs( fMag ) )
		iIndex += 1
	endWhile
EndFunction


;
; EVENTS & STATES ---------------------------------------------------------------------------------
;

State onAddAttributes
	Event OnUpdate()
		if ( !RRControl.isActive() )
			deactivate()
			return
		endIf

		addAttributesSpells()

		UnregisterForUpdate()
		GotoState("Inactive")
	EndEvent
EndState

State Inactive
	Event OnUpdate()
		; nop
	EndEvent
EndState

Event OnUpdate()
	; nop
EndEvent
