Scriptname RRFatigueBowstringScript extends Quest  
{Realistic Regeneration : Fatigue - Nocking}

;
; GLOBAL VARIABLES --------------------------------------------------------------------------------
;

; Game
Actor Property PlayerREF Auto

; Scripts
RRControlScript Property RRControl Auto

GlobalVariable Property RRRelaxArrowHealEnabled    Auto
GlobalVariable Property RRRelaxArrowMagickaEnabled Auto
GlobalVariable Property RRRelaxArrowStaminaEnabled Auto

GlobalVariable Property RRRelaxArrowHealValue    Auto
GlobalVariable Property RRRelaxArrowMagickaValue Auto
GlobalVariable Property RRRelaxArrowStaminaValue Auto

; Timers
GlobalVariable Property RRUpdateFatigueBowstringAt Auto

; Hide In UI
GlobalVariable Property RRHideInUI Auto

;
; SPELLS ------------------------------------------------------------------------------------------
;

Spell Property RRRelaxArrowHealSpell    Auto
Spell Property RRRelaxArrowMagickaSpell Auto
Spell Property RRRelaxArrowStaminaSpell Auto

;
; PRIVATE VARIABLES -------------------------------------------------------------------------------
;

GlobalVariable[] gEnabledFatiguesList
GlobalVariable[] gFatiguesValuesList
Spell[]          sFatiguesSpellsList

int Property HideInUI = 0x00008000 autoReadOnly

;
; FUNCTIONS ---------------------------------------------------------------------------------------
;

Function initializeAll()
	initializeVariables()
EndFunction

Function reset()
	deactivate()

	if ( RRControl.isActive() )
		activate()
	endIf
EndFunction

Function activate()
	removeAllSpells()
	setPlayerSettings()
	GotoState( "onNockFatigue" )
	RegisterForSingleUpdate( RRUpdateFatigueBowstringAt.GetValue() )
EndFunction

Function deactivate()
	removeAllSpells()
	UnregisterForAnimationEvent( PlayerREF, "arrowAttach" )
	UnregisterForAnimationEvent( PlayerREF, "arrowDetach" )
	UnregisterForUpdate()
	GotoState( "Inactive" )
EndFunction

Function initializeVariables()
	gEnabledFatiguesList = new GlobalVariable[3]
	gEnabledFatiguesList[0] = RRRelaxArrowHealEnabled
	gEnabledFatiguesList[1] = RRRelaxArrowMagickaEnabled
	gEnabledFatiguesList[2] = RRRelaxArrowStaminaEnabled

	gFatiguesValuesList = new GlobalVariable[3]
	gFatiguesValuesList[0] = RRRelaxArrowHealValue
	gFatiguesValuesList[1] = RRRelaxArrowMagickaValue
	gFatiguesValuesList[2] = RRRelaxArrowStaminaValue

	sFatiguesSpellsList = new Spell[3]
	sFatiguesSpellsList[0] = RRRelaxArrowHealSpell
	sFatiguesSpellsList[1] = RRRelaxArrowMagickaSpell
	sFatiguesSpellsList[2] = RRRelaxArrowStaminaSpell
EndFunction

Function removeAllSpells()
	RRControl.removeAllSpellsFromList( sFatiguesSpellsList )
EndFunction

Function setPlayerSettings()
	int iIndex = 0

	While ( iIndex < gEnabledFatiguesList.Length )
		Spell aSpell = sFatiguesSpellsList[iIndex]
		float fMagnitude = gFatiguesValuesList[iIndex].GetValue()

		setEffectMagnitude( aSpell, fMagnitude )
		iIndex += 1
	EndWhile
EndFunction

Function setEffectMagnitude( Spell aSpell, float fMagnitude )
	MagicEffect spellEffect = aSpell.GetNthEffectMagicEffect( 0 )

	if ( RRHideInUI.GetValueInt() == 1 )
		if ( !spellEffect.IsEffectFlagSet( HideInUI ) )
			spellEffect.SetEffectFlag( HideInUI )
		endIf
	else
		if ( spellEffect.IsEffectFlagSet( HideInUI ) )
			spellEffect.ClearEffectFlag( HideInUI )
		endIf
	endIf

	aSpell.SetNthEffectMagnitude( 0, fMagnitude )
EndFunction

Function registerFatiguesEvents()
	RegisterForAnimationEvent( PlayerREF, "arrowAttach" )
	RegisterForAnimationEvent( PlayerREF, "arrowDetach" )

	If ( RRControl.isDebug() )
		Debug.Notification( "Added Bow Fatigue" )
	EndIf
EndFunction

Function startFatiguesSpells()
	int iIndex = 0
	bool bDebug = RRControl.isDebug()

	While ( iIndex < gEnabledFatiguesList.Length )
		if ( gEnabledFatiguesList[iIndex].GetValueInt() == 1 )
			PlayerREF.AddSpell( sFatiguesSpellsList[iIndex], bDebug )
		endIf

		iIndex += 1
	EndWhile
EndFunction

Function stopFatiguesSpells()
	removeAllSpells()
EndFunction

;
; isEnabled() - returns true if we really want to enable fatigue when nock
;
bool Function isEnabled()
	If !RRControl.isActive()
		return false
	EndIf

	int iIndex = 0

	While iIndex < gEnabledFatiguesList.Length
		if gEnabledFatiguesList[iIndex].GetValueInt() == 1
			return true
		endIf

		iIndex += 1
	EndWhile

	return false
EndFunction


;
; EVENTS & STATES ---------------------------------------------------------------------------------
;

State OnNockFatigue
	Event OnUpdate()
		If isEnabled()
			registerFatiguesEvents()
		else
			deactivate()
			return
		EndIf

		UnregisterForUpdate()
		GotoState("Inactive")
	EndEvent
EndState

State Inactive
	Event OnUpdate()
		; nop
	EndEvent
EndState

Event OnUpdate()
	; nop
EndEvent

Event OnAnimationEvent( ObjectReference akSource, string asEventName )
	if ( akSource == PlayerREF )
		if ( asEventName == "arrowAttach" )

			startFatiguesSpells()

		elseif ( asEventName == "arrowDetach" )

			stopFatiguesSpells()

		else
			return
		endIf
	endIf
EndEvent
