Scriptname RRGameSettingsScript extends Quest  
{Realistic Regeneration : Game Settings}


;
; GLOBAL VARIABLES --------------------------------------------------------------------------------
;

; Scripts
RRControlScript Property RRControl Auto

; Timers
GlobalVariable Property RRGameSettingsAt Auto

GlobalVariable Property RRGSfSprintStaminaDrainMult Auto
GlobalVariable Property RRGSfSprintStaminaWeightBase Auto
GlobalVariable Property RRGSfSprintStaminaWeightMult Auto
GlobalVariable Property RRGSfStaminaAttackWeaponMult Auto
GlobalVariable Property RRGSfPowerAttackStaminaPenalty Auto
GlobalVariable Property RRGSfStaminaBashBase Auto
GlobalVariable Property RRGSfStaminaPowerBashBase Auto
GlobalVariable Property RRGSfStaminaBlockDmgMult Auto

; -- added in 3.10
GlobalVariable Property RRGSfMagicCasterPCSkillCostBase Auto
GlobalVariable Property RRGSfMagicCasterPCSkillCostMult Auto
GlobalVariable Property RRGSfMagicCasterSkillCostBase Auto
GlobalVariable Property RRGSfMagicCasterSkillCostMult Auto


;
; PRIVATE VARIABLES -------------------------------------------------------------------------------
;

String[] Property sGameSettingsList Auto Hidden
GlobalVariable[] Property gvModifiersList Auto Hidden


;
; FUNCTIONS ---------------------------------------------------------------------------------------
;


Function initializeAll()
	initializeVariables()
EndFunction

Function updateAll(int iOption = 0)
	initializeVariables()
	ReadGameSettings( iOption )
EndFunction

Function reset()
	deactivate()

	if ( RRControl.isActive() )
		activate()
	endIf
EndFunction

Function activate()
	GotoState( "onGameSettings" )
	RegisterForSingleUpdate( RRGameSettingsAt.GetValue() )
EndFunction

Function deactivate()
	UnregisterForUpdate()
	GotoState( "Inactive" )
EndFunction

Function initializeVariables()
	sGameSettingsList = new String[12]
	sGameSettingsList[0] = "fSprintStaminaDrainMult"
	sGameSettingsList[1] = "fSprintStaminaWeightBase"
	sGameSettingsList[2] = "fSprintStaminaWeightMult"
	sGameSettingsList[3] = "fStaminaAttackWeaponMult"
	sGameSettingsList[4] = "fPowerAttackStaminaPenalty"
	sGameSettingsList[5] = "fStaminaBashBase"
	sGameSettingsList[6] = "fStaminaPowerBashBase"
	sGameSettingsList[7] = "fStaminaBlockDmgMult"
	sGameSettingsList[8] = "fMagicCasterPCSkillCostBase"
	sGameSettingsList[9] = "fMagicCasterPCSkillCostMult"
	sGameSettingsList[10] = "fMagicCasterSkillCostBase"
	sGameSettingsList[11] = "fMagicCasterSkillCostMult"

	gvModifiersList = new GlobalVariable[12]
	gvModifiersList[0] = RRGSfSprintStaminaDrainMult
	gvModifiersList[1] = RRGSfSprintStaminaWeightBase
	gvModifiersList[2] = RRGSfSprintStaminaWeightMult
	gvModifiersList[3] = RRGSfStaminaAttackWeaponMult
	gvModifiersList[4] = RRGSfPowerAttackStaminaPenalty
	gvModifiersList[5] = RRGSfStaminaBashBase
	gvModifiersList[6] = RRGSfStaminaPowerBashBase
	gvModifiersList[7] = RRGSfStaminaBlockDmgMult
	gvModifiersList[8] = RRGSfMagicCasterPCSkillCostBase
	gvModifiersList[9] = RRGSfMagicCasterPCSkillCostMult
	gvModifiersList[10] = RRGSfMagicCasterSkillCostBase
	gvModifiersList[11] = RRGSfMagicCasterSkillCostMult
EndFunction

Function ReadGameSettings(int iBaseIndex = 0)
	Int iIndex = iBaseIndex
	Int iEntries = gvModifiersList.Length

	If RRControl.isDebug()
		Debug.Notification("Found " + iEntries + " Game Settings entries")
	EndIf

	While iIndex < iEntries
		gvModifiersList[iIndex].SetValue( Game.GetGameSettingFloat( sGameSettingsList[iIndex] ) )
		iIndex += 1
	EndWhile
EndFunction

; -- added in 3.11
Function RestoreGameSettingsDefaults()
	Game.SetGameSettingFloat( "fSprintStaminaDrainMult", 7.0 )
	Game.SetGameSettingFloat( "fSprintStaminaWeightBase", 1.0 )
	Game.SetGameSettingFloat( "fSprintStaminaWeightMult", 0.02 )
	Game.SetGameSettingFloat( "fStaminaAttackWeaponMult", 1.0 )
	Game.SetGameSettingFloat( "fPowerAttackStaminaPenalty", 2.0 )
	Game.SetGameSettingFloat( "fStaminaBashBase", 35.0 )
	Game.SetGameSettingFloat( "fStaminaPowerBashBase", 55.0 )
	Game.SetGameSettingFloat( "fStaminaBlockDmgMult", 0.25 )
EndFunction

Function UpdateGameSettings()
	Int iIndex = 0
	Int iEntries = gvModifiersList.Length

	While iIndex < iEntries
		Float fValue = gvModifiersList[iIndex].GetValue()

;		If RRControl.isDebug()
;			Debug.Notification("Set " + sGameSettingsList[iIndex] + " to " + fValue)
;		EndIf

		Game.SetGameSettingFloat( sGameSettingsList[iIndex], fValue )
		iIndex += 1
	EndWhile

	if RRControl.isDebug()
		Debug.Notification("Updated " + iEntries + " Game Settings entries")
	EndIf
EndFunction


;
; EVENTS & STATES ---------------------------------------------------------------------------------
;

State onGameSettings
	Event OnUpdate()
		if ( !RRControl.isActive() )
			deactivate()
			return
		endIf

		UpdateGameSettings()

		UnregisterForUpdate()
		GotoState("Inactive")
	EndEvent
EndState

State Inactive
	Event OnUpdate()
		; nop
	EndEvent
EndState

Event OnUpdate()
	; nop
EndEvent
