Scriptname RRInGameScript extends Quest  
{Realistic Regeneration : In Game Script (werewolf)}

;
; GLOBAL VARIABLES --------------------------------------------------------------------------------
;

; Game
Race    Property WerewolfBeastRace  Auto
Actor   Property PlayerREF          Auto

; Scripts
RRControlScript           Property RRControl        Auto

; -- added in 3.10
RRBoostersScript          Property RRBoosters       Auto
RRFatigueBowstringScript  Property RRFatigueBow     Auto
RRGameSettingsScript      Property RRGameSettings   Auto
; -- added in 3.11
GlobalVariable            Property RRWerewolfRestoreGS Auto


; Default rates
GlobalVariable Property RRHealRateIndex Auto
GlobalVariable Property RRHealRate Auto

GlobalVariable Property RRMagickaRateIndex Auto
GlobalVariable Property RRMagickaRate Auto

GlobalVariable Property RRStaminaRateIndex Auto
GlobalVariable Property RRStaminaRate Auto

; Werewolf rates
GlobalVariable Property RRWerewolfIndex Auto

GlobalVariable Property RRWerewolfHealRateIndex Auto
GlobalVariable Property RRWerewolfHealRateValue Auto

GlobalVariable Property RRWerewolfMagickaRateIndex Auto
GlobalVariable Property RRWerewolfMagickaRateValue Auto

GlobalVariable Property RRWerewolfStaminaRateIndex Auto
GlobalVariable Property RRWerewolfStaminaRateValue Auto

; Timers
GlobalVariable Property RRWerewolfUpdateInterval Auto

; Constants
GlobalVariable Property RRFunctionNoHeal Auto
GlobalVariable Property RRFunctionDefault Auto
GlobalVariable Property RRFunctionUserDefined Auto
GlobalVariable Property RRFunctionVanilla Auto

;
; PRIVATE VARIABLES -------------------------------------------------------------------------------
;

bool beastFormValuesUpdated

float Property   defaultWerewolfHealRate    = 0.0   autoReadOnly
float Property   defaultWerewolfMagickaRate = 5.0   autoReadOnly
float Property   defaultWerewolfStaminaRate = 20.0  autoReadOnly

string[]         avRatesList
string[]         avNamesList
float[]          defaultRatesList

GlobalVariable[] wwIndexList
GlobalVariable[] wwValueList

GlobalVariable[] defaultIndexList
GlobalVariable[] defaultValueList


;
; FUNCTIONS ---------------------------------------------------------------------------------------
;

Function initializeAll()
	beastFormValuesUpdated = false

	avNamesList = new string[3]
	avNamesList[0] = "Health"
	avNamesList[1] = "Magicka"
	avNamesList[2] = "Stamina"

	avRatesList = new string[3]
	avRatesList[0] = "HealRate"
	avRatesList[1] = "MagickaRate"
	avRatesList[2] = "StaminaRate"

	defaultRatesList = new float[3]
	defaultRatesList[0] = defaultWerewolfHealRate
	defaultRatesList[1] = defaultWerewolfMagickaRate
	defaultRatesList[2] = defaultWerewolfStaminaRate

	wwIndexList = new GlobalVariable[3]
	wwIndexList[0] = RRWerewolfHealRateIndex
	wwIndexList[1] = RRWerewolfMagickaRateIndex
	wwIndexList[2] = RRWerewolfStaminaRateIndex

	wwValueList = new GlobalVariable[3]
	wwValueList[0] = RRWerewolfHealRateValue
	wwValueList[1] = RRWerewolfMagickaRateValue
	wwValueList[2] = RRWerewolfStaminaRateValue

	defaultIndexList = new GlobalVariable[3]
	defaultIndexList[0] = RRHealRateIndex
	defaultIndexList[1] = RRMagickaRateIndex
	defaultIndexList[2] = RRStaminaRateIndex

	defaultValueList = new GlobalVariable[3]
	defaultValueList[0] = RRHealRate
	defaultValueList[1] = RRMagickaRate
	defaultValueList[2] = RRStaminaRate
endFunction

Function reset()
	if ( RRControl.isActive() )
		activate()
	else
		deactivate()
	endif
endFunction

Function activate()
	if ( isEnabled() )
		processBeastForms()

		GoToState( "Polling" )
		RegisterForSingleUpdate( 0.5 )
	else
		deactivate()
	endIf
EndFunction

Function deactivate()
	UnRegisterForUpdate()
	GoToState("Inactive")
endFunction

bool Function isEnabled()
	return RRWerewolfIndex.GetValueInt() == 1
endFunction

Function processBeastForms()
	if ( isWerewolfForm() )
		if ( beastFormValuesUpdated )
			; nothing to do
			return
		endIf

		; clear all effects
		RRControl.removeAllSpells()
		RRBoosters.removeAllSpells()
		RRFatigueBow.removeAllSpells()

		If RRWerewolfRestoreGS.GetValueInt() == 1
			RRGameSettings.RestoreGameSettingsDefaults()
		EndIf

		if ( isEnabled() )
			int iIndex = 0
			while ( iIndex < avRatesList.Length )
				setWerewolfRate( iIndex )

				iIndex += 1
			endWhile
		else
			; Realistic Regeneration first was enabled
			; and later it was disabled
			restoreRates()
		endIf

		beastFormValuesUpdated = true

		return
	endIf

	; player character is NOT in werewolf anymore
	;
	if ( beastFormValuesUpdated )
		beastFormValuesUpdated = false

		restoreRates()

		RRControl.reset()
		RRBoosters.reset()
		RRFatigueBow.reset()

		If RRWerewolfRestoreGS.GetValueInt() == 1
			RRGameSettings.reset()
		EndIf
	endIf
endFunction

bool Function isWerewolfForm()
	return PlayerREF.GetRace() == WerewolfBeastRace
endFunction

Function setWerewolfRate( int iIndex )
	string rateName = avRatesList[iIndex]
	float rateValue = 0.0
	int fType = wwIndexList[iIndex].GetValueInt()

	if ( fType == RRFunctionNoHeal.GetValueInt() )
		rateValue = 0.0
	elseif ( fType == RRFunctionDefault.GetValueInt() )
		rateValue = defaultRatesList[iIndex]
	elseif ( fType == RRFunctionUserDefined.GetValueInt() )
		rateValue = wwValueList[iIndex].GetValue()
	elseif ( fType == RRFunctionVanilla.GetValueInt() )
		return
	else
		return
	endIf

	PlayerREF.ForceAV( rateName, rateValue )

	if ( RRControl.isDebug() )
		Debug.Notification( "Set [" + rateName + "]: " + rateValue )
	endIf
endFunction

Function restoreRates()
	int iIndex = 0
	while ( iIndex < avRatesList.Length )
		restoreDefaultRate( iIndex )

		iIndex += 1
	endWhile
endFunction

Function restoreDefaultRate( int iIndex )
	string avName = avNamesList[iIndex]
	int iDefaultType = defaultIndexList[iIndex].GetValueInt()
	float fDefaultRate = getDefaultRate( iDefaultType, defaultValueList[iIndex], avName )

	string rateName = avRatesList[iIndex]

	PlayerREF.ForceAV( rateName, fDefaultRate )

	if ( RRControl.isDebug() )
		Debug.Notification( "Restored [" + rateName + "]: " + fDefaultRate )
	endIf
endFunction

float Function getDefaultRate( int fType, GlobalVariable gvValue, string avName )
	if ( fType == RRFunctionNoHeal.GetValueInt() )
		return 0.0
	elseif ( fType == RRFunctionDefault.GetValueInt() )
		return RRControl.getDefaultRate( avName )
	elseif ( fType == RRFunctionUserDefined.GetValueInt() )
		return gvValue.GetValue()
	elseif ( fType == RRFunctionVanilla.GetValueInt() )
		int iIndex = avNamesList.Find( avName )
		return PlayerREF.GetBaseActorValue( avRatesList[iIndex] )
	else
		if ( RRControl.isDebug () )
			Debug.Notification( "RR werewolf getDefaultRate() wrong value: " + fType )
		endIf

		int iIndex = avNamesList.Find( avName )
		return PlayerREF.GetBaseActorValue( avRatesList[iIndex] )
	endIf
EndFunction

;
; STATES & EVENTS ---------------------------------------------------------------------------------
;

State Polling
	Event OnUpdate()
		if ( RRControl.isActive() )
			processBeastForms()
			RegisterForSingleUpdate( RRWerewolfUpdateInterval.GetValue() )
		else
			deactivate()
		endif
	endEvent
endState

Event OnUpdate()
	; nop
endEvent

State Inactive
	Event OnUpdate()
		; nop
	EndEvent
EndState
