Scriptname RRSkyUIConfigMenuScript extends RRConfigAttributesScript  
{Realistic Regeneration for SkyUI : Children Script}

; +-RRSkyUIConfigParentScript
; +-+-RRConfigHelpScript
;   +-+-RRConfigSlidersScript
;     +-+-RRConfigTemplatesScript
;       +-+-RRConfigMiscScript
;         +-+-RRConfigBeastFormScript
;           +-+-RRConfigCombatScript
;             +-+-RRConfigBoostersScript
;               +-+-RRConfigAttributesScript
;                 +-+-RRSkyUIConfigMenuScript



; SCRIPT VERSION ----------------------------------------------------------------------------------

int function GetVersion()
	return 311
endFunction

;
; EVENTS : INITIALIZATION -------------------------------------------------------------------------
;

; @overrides SKI_ConfigBase
event OnConfigInit()
	initializeAll()
endEvent

; @implements SKI_ConfigBase
event OnConfigRegister()
	if (RRControl.isActive())
		Debug.Notification( "$RRnotifyModEnabled" )
	else
		Debug.Notification( "$RRnotifyModDisabled" )
	endIf
endEvent

;
; EVENTS : SkyUI ----------------------------------------------------------------------------------
;

event OnVersionUpdate(int a_version)
	; a_version is the new version, CurrentVersion is the old version
	initializeAll()
	RRSkyUILib.doVersionUpdate( CurrentVersion, a_version )
endEvent

event OnConfigClose()
	resetAll()
endEvent

; @implements SKI_ConfigBase
event OnPageReset(string a_page)
	if (a_page == "")
		return
	else
		UnloadCustomContent()
	endIf

	if (a_page == "$Options")

		updateOptionsFlags()
		RRSkyUILib.updateRatesAndAttrs()
		generateOptionsPage()

	elseif (a_page == "$RRStayAndSit")

		updateStayAndSitFlags()
		generateStayAndSitPage()

	elseif (a_page == "$RRSleepAndWait")

		updateSleepAndWaitFlags()
		generateSleepAndWaitPage()

	elseif (a_page == "$RRRunAndSprint")

		updateRunAndSprintFlags()
		generateRunAndSprintPage()

	elseif (a_page == "$RRPotionsAndFood")

		updatePotionsAndFoodFlags()
		generatePotionsAndFoodPage()

	elseif (a_page == "$RRItemsAndSpells")

		updateItemsAndSpellsFlags()
		generateItemsAndSpellsPage()

	elseif (a_page == "$RRCombat")

		updateCombatFlags()
		generateCombatPage()

	elseif (a_page == "$RRBeastForm")

		updateBeastFormFlags()
		generateBeastFormPage()

	elseif (a_page == "$RRCourageAndFatigue")

		updateCourageAndFatigueFlags()
		generateCourageAndFatiguePage()

	elseif (a_page == "$RRAttributes")

		updateAttributesFlags()
		generateAttributesPage()

	elseif (a_page == "$RRTemplates")

		updateTemplatesFlags()
		generateTemplatesPage()

	elseif (a_page == "$RRMiscOptions")

		updateMiscOptionsFlags()
		generateMiscOptionsPage()

	elseif (a_page == "$RRSlidersOptions")

		updateSlidersOptionsFlags()
		generateSlidersOptionsPage()

	elseif (a_page == "$Help")

		generateHelpPage()

	endIf
endEvent


;
; FUNCTIONS : Intialization -----------------------------------------------------------------------
;

Function initializeAll()
	setPages()
	parent.initializeAll()

	RRSkyUILib.setSaveSlot()
	RRSkyUILib.setRatesAndAttrs()
endFunction

Function setPages()
	Pages = new string[14]
	Pages[0] = "$Options"
	Pages[1] = "$RRStayAndSit"
	Pages[2] = "$RRSleepAndWait"
	Pages[3] = "$RRRunAndSprint"
	Pages[4] = "$RRPotionsAndFood"
	Pages[5] = "$RRItemsAndSpells"
	Pages[6] = "$RRCombat"
	Pages[7] = "$RRBeastForm"
	Pages[8] = "$RRCourageAndFatigue"
	Pages[9] = "$RRAttributes"
	Pages[10] = "$RRTemplates"
	Pages[11] = "$RRMiscOptions"
	Pages[12] = "$RRSlidersOptions"
	Pages[13] = "$Help"
endFunction



;
; PAGES -------------------------------------------------------------------------------------------
;
; PAGE Options ------------------------------------------------------------------------------------
;

Function generateOptionsPage()
	SetCursorFillMode(TOP_TO_BOTTOM)

	AddToggleOptionST("MOD_ENABLED", "$RRActivateMod", RREnabled.GetValueInt())
	AddEmptyOption()

	AddHeaderOption("$RREnableOptions", OPTION_FLAG_NONE)
	AddToggleOptionST("HEALTH_ENABLED", "$Health", RRHealEnabled.GetValueInt(), _modEnabledFlags)
	AddToggleOptionST("MANA_ENABLED", "$Magicka", RRMagickaEnabled.GetValueInt(), _modEnabledFlags)
	AddToggleOptionST("STAMINA_ENABLED", "$Stamina", RRStaminaEnabled.GetValueInt(), _modEnabledFlags)
	AddEmptyOption()

	AddHeaderOption("$RRSaveAndLoad", OPTION_FLAG_NONE)
	AddTextOptionST("TEMPLATE_SAVE_SETTINGS", "$RRSaveSettings", "$Apply", _modEnabledFlags)
	AddTextOptionST("TEMPLATE_SAVE_SLOT", "$RRSlot", _saveSlotDesc, _modEnabledFlags)
	AddTextOptionST("TEMPLATE_LOAD_SETTINGS", "$RRLoadSettings", "$Apply", _modEnabledFlags)

	SetCursorPosition(1)

	AddTextOptionST("MOD_VERSION", "$RRVersion", RRMaintainance.getRRVersion())
	AddEmptyOption()

	AddHeaderOption("$RRCurRatesAndAttrs", OPTION_FLAG_NONE)

	float[] fStatsList = RRSkyUILib.getStatsList()
	AddTextOptionST("CURRENT_X_HEALTH", "$Health", Math.Floor(fStatsList[0]), OPTION_FLAG_NONE)
	AddTextOptionST("CURRENT_X_MAGICKA", "$Magicka", Math.Floor(fStatsList[1]), OPTION_FLAG_NONE)
	AddTextOptionST("CURRENT_X_STAMINA", "$Stamina", Math.Floor(fStatsList[2]), OPTION_FLAG_NONE)
	AddTextOptionST("CURRENT_MEASURE", "", "$RRAttribute", OPTION_FLAG_NONE)
	AddEmptyOption()

	if ( RREnabled.GetValueInt() == 1)
		AddToggleOptionST("RR_UNINSTALL", "$RRUninstall", RRUninstall.GetValueInt(), OPTION_FLAG_DISABLED)
	else
		AddToggleOptionST("RR_UNINSTALL", "$RRUninstall", RRUninstall.GetValueInt(), OPTION_FLAG_NONE)
	endIf

	AddEmptyOption()
	AddToggleOptionST("DEBUG_ENABLED", "$RRDebug", RRDebugEnabled.GetValueInt(), _modEnabledFlags)
endFunction

Function updateOptionsFlags()
	updateModEnabledFlags()
	updateSaveAndLoadFlags()
endFunction

Function updateSaveAndLoadFlags()
	if ( RRSkyUILib.isSaveSlotFull() )
		_saveSlotDesc = "$RRFilled"
	else
		_saveSlotDesc = "$RREmpty"
	endIf
endFunction

Function RatesAndAttrsShowCase()
	iStatsIndex += 3

	if ( iStatsIndex >= 12 )
		iStatsIndex = 0
	endIf

	float[] fStatsList = RRSkyUILib.getStatsList()

	if ( iStatsIndex == 0 )
		SetTextOptionValueST("$RRAttribute", false, "CURRENT_MEASURE" )
		SetTextOptionValueST( Math.Floor(fStatsList[0]), false, "CURRENT_X_HEALTH" )
		SetTextOptionValueST( Math.Floor(fStatsList[1]), false, "CURRENT_X_MAGICKA" )
		SetTextOptionValueST( Math.Floor(fStatsList[2]), false, "CURRENT_X_STAMINA" )
		return
	elseif ( iStatsIndex == 3 )
		SetTextOptionValueST( "$RRRate", false, "CURRENT_MEASURE" )
	elseif ( iStatsIndex == 6 )
		SetTextOptionValueST( "$RRPerSecond", false, "CURRENT_MEASURE" )
	elseif ( iStatsIndex == 9 )
		SetTextOptionValueST( "$RRCombatPerSecond", false, "CURRENT_MEASURE" )
	endIf

	SetTextOptionValueST( fStatsList[iStatsIndex], false, "CURRENT_X_HEALTH" )
	SetTextOptionValueST( fStatsList[iStatsIndex + 1], false, "CURRENT_X_MAGICKA" )
	SetTextOptionValueST( fStatsList[iStatsIndex + 2], false, "CURRENT_X_STAMINA" )
endFunction

Function updateInterface()
	if ( RREnabled.GetValueInt() == 1)
		SetOptionFlagsST(OPTION_FLAG_DISABLED, false, "RR_UNINSTALL")
	else
		SetOptionFlagsST(OPTION_FLAG_NONE, false, "RR_UNINSTALL")
	endIf

	SetToggleOptionValueST(RRHealEnabled.GetValueInt(), false, "HEALTH_ENABLED")
	SetToggleOptionValueST(RRMagickaEnabled.GetValueInt(), false, "MANA_ENABLED")
	SetToggleOptionValueST(RRStaminaEnabled.GetValueInt(), false, "STAMINA_ENABLED")

	SetOptionFlagsST(_modEnabledFlags, false, "HEALTH_ENABLED")
	SetOptionFlagsST(_modEnabledFlags, false, "MANA_ENABLED")
	SetOptionFlagsST(_modEnabledFlags, false, "STAMINA_ENABLED")

	SetOptionFlagsST(_modEnabledFlags, false, "TEMPLATE_SAVE_SETTINGS")
	SetOptionFlagsST(_modEnabledFlags, false, "TEMPLATE_SAVE_SLOT")
	SetOptionFlagsST(_modEnabledFlags, false, "TEMPLATE_LOAD_SETTINGS")

	SetOptionFlagsST(_modEnabledFlags, false, "DEBUG_ENABLED")
EndFunction

;
; STATE Options -----------------------------------------------------------------------------------
;

state MOD_VERSION ; TEXT
	event OnHighlightST()
		SetInfoText("$RRVersionInfo")
	endEvent
endState

state CURRENT_X_HEALTH ; TEXT
	event OnSelectST()
		RatesAndAttrsShowCase()
	endEvent

	event OnHighlightST()
		SetInfoText("$RRhintCurRatesAndAttrs")
	endEvent
endState

state CURRENT_X_MAGICKA ; TEXT
	event OnSelectST()
		RatesAndAttrsShowCase()
	endEvent

	event OnHighlightST()
		SetInfoText("$RRhintCurRatesAndAttrs")
	endEvent
endState

state CURRENT_X_STAMINA ; TEXT
	event OnSelectST()
		RatesAndAttrsShowCase()
	endEvent

	event OnHighlightST()
		SetInfoText("$RRhintCurRatesAndAttrs")
	endEvent
endState

state CURRENT_MEASURE ; TEXT
	event OnSelectST()
		RatesAndAttrsShowCase()
	endEvent

	event OnHighlightST()
		SetInfoText("$RRhintMeasure")
	endEvent
endState

state DEBUG_ENABLED ; TOGGLE
	event OnSelectST()
		RRSkyUILib.toggle(RRDebugEnabled)
		SetToggleOptionValueST(RRDebugEnabled.GetValueInt())
	endEvent

	event OnHighlightST()
		SetInfoText("$RRhintDebugEnabled")
	endEvent
endState

;
; STATE Options : Toggles -------------------------------------------------------------------------
;

state MOD_ENABLED ; TOGGLE
	event onSelectST()
		RRSkyUILib.toggle(RREnabled)
		SetToggleOptionValueST(RREnabled.GetValueInt())
		updateOptionsFlags()
		updateInterface()
	endEvent

	event OnHighlightST()
		SetInfoText("$RRhintActivateMod")
	endEvent
endState

state HEALTH_ENABLED ; TOGGLE
	event OnSelectST()
		RRSkyUILib.toggle(RRHealEnabled)
		SetToggleOptionValueST(RRHealEnabled.GetValueInt())
	endEvent
	
	event OnHighlightST()
		SetInfoText("$RRhintHealthEnabled")
	endEvent
endState

state MANA_ENABLED ; TOGGLE
	event OnSelectST()
		RRSkyUILib.toggle(RRMagickaEnabled)
		SetToggleOptionValueST(RRMagickaEnabled.GetValueInt())
	endEvent
	
	event OnHighlightST()
		SetInfoText("$RRhintMagickaEnabled")
	endEvent
endState

state STAMINA_ENABLED ; TOGGLE
	event OnSelectST()
		RRSkyUILib.toggle(RRStaminaEnabled)
		SetToggleOptionValueST(RRStaminaEnabled.GetValueInt())
	endEvent
	
	event OnHighlightST()
		SetInfoText("$RRhintStaminaEnabled")
	endEvent
endState

state RR_UNINSTALL ; TOGGLE
	event OnSelectST()
		RRSkyUILib.toggle(RRUninstall)
		SetToggleOptionValueST(RRUninstall.GetValueInt())
	endEvent
	
	event OnHighlightST()
		SetInfoText("$RRhintUninstall")
	endEvent
endState

;
; STATE Options - Save & Load : Text --------------------------------------------------------------
;

state TEMPLATE_SAVE_SETTINGS ; Text
	event OnSelectST()
		if ( RRSkyUILib.isSaveSlotFull() )
			if ( ShowMessage( "$RROverwriteWarning" ) )
				RRSkyUILib.saveSettings()
			endIf
		else
			RRSkyUILib.saveSettings()
		endIf

		updateSaveAndLoadFlags()
		SetTextOptionValueST( _saveSlotDesc, false, "TEMPLATE_SAVE_SLOT" )
	endEvent

	event OnHighlightST()
		SetInfoText("$RRhintSettings")
	endEvent
endState

state TEMPLATE_SAVE_SLOT ; Text
	event OnHighlightST()
		SetInfoText("$RRhintSlot")
	endEvent
endState

state TEMPLATE_LOAD_SETTINGS ; Text
	event OnSelectST()
		if ( RRSkyUILib.isSaveSlotFull() )
			if ( ShowMessage("$RRTemplateWarning") )
				RRSkyUILib.loadSettings()
				updateOptionsFlags()
				updateInterface()
			endIf
		else
			ShowMessage( "$RRNotFoundMessage", false )
		endIf
	endEvent

	event OnHighlightST()
		SetInfoText( "$RRhintLoadSettings" )
	endEvent
endState
