Scriptname RRWhenSleepScript extends Quest
{Recalculate Health, Magicka and Stamina after a sleep}

;
; GLOBAL VARIABLES --------------------------------------------------------------------------------
;

; Game
Actor Property PlayerREF Auto

; Scripts
RRControlScript Property RRControl Auto

; Timers
GlobalVariable Property RRUpdateSleepAt Auto

; Constants
GlobalVariable Property RRFunctionNoHeal Auto
GlobalVariable Property RRFunctionDefault Auto
GlobalVariable Property RRFunctionUserDefined Auto
GlobalVariable Property RRFunctionVanilla Auto


;
; PRIVATE VARIABLES -------------------------------------------------------------------------------
;

float       _sleepStartTime
float       _sleepEndTime
bool        _isWasSleep
bool        _isInterrupted

string[]    avNamesList
float[]     beforeSleepValuesList
float[]     afterSleepValuesList

int         iNO_HEAL
int         iDEFAULT
int         iDEFINED
int         iVANILLA

;
; FUNCTIONS ---------------------------------------------------------------------------------------
;

Function initializeAll()
	avNamesList = new string[3]
	avNamesList[0] = "Health"
	avNamesList[1] = "Magicka"
	avNamesList[2] = "Stamina"

	beforeSleepValuesList = new float[3]
	beforeSleepValuesList[0] = 0.0
	beforeSleepValuesList[1] = 0.0
	beforeSleepValuesList[2] = 0.0

	afterSleepValuesList = new float[3]
	afterSleepValuesList[0] = 0.0
	afterSleepValuesList[1] = 0.0
	afterSleepValuesList[2] = 0.0

	iNO_HEAL = RRFunctionNoHeal.GetValueInt()
	iDEFAULT = RRFunctionDefault.GetValueInt()
	iDEFINED = RRFunctionUserDefined.GetValueInt()
	iVANILLA = RRFunctionVanilla.GetValueInt()

	if ( iDEFAULT == 0 )
		Debug.Notification( "RR (sleep): incorrect installation!" )
	endIf
endFunction

Function reset()
	if ( RRControl.isActive() )
		activate()
	else
		deactivate()
	endif
endFunction

Function activate()
	bool bEnable = false
	int iIndex = 0
	while ( iIndex < avNamesList.Length )
		if ( !bEnable )
			if ( isSleepEnabledFor( avNamesList[iIndex] ) )
				bEnable = true
			endIf
		endIf

		iIndex += 1
	endWhile

	if ( bEnable )
		RegisterForMenu("Sleep/Wait Menu")
		RegisterForSleep()
	else
		deactivate()
	endIf
endFunction

Function deactivate()
	UnregisterForMenu("Sleep/Wait Menu")
	UnregisterForSleep()
endFunction

Function setAVAfterSleep( int avIndex, float period )
	string avName = avNamesList[avIndex]
	int fType = RRControl.getFunctionType( "Sleep", avName )
	float rv = 0.0

	if ( fType == iNO_HEAL )
		; nop
	elseif ( fType == iDEFAULT )
		rv = getModificator( period, RRControl.getRateValue( avName ) )
	elseif ( fType == iDEFINED )
		rv = getModificator( period, RRControl.getRateByType( "Sleep", avName ) )
	elseif ( fType == iVANILLA )
		rv = getModificator( period, RRControl.getCurrentRateByIndex( avIndex ) )
	else
		return
	endIf

	if ( RRControl.isDebug() )
		Debug.Notification( "Gained after a sleep [" + avName + "]: " + rv )
	endIf

	afterSleepValuesList[avIndex] = beforeSleepValuesList[avIndex] + rv
endFunction

; Count how much HP we able to restore
; when currentRate == 0 we dont regenerate at all
;
float Function getModificator( float fPeriod, float currentRate )
	; Rate is always positive number: our functions returns -1.0 if smth goes wrong
	If currentRate < 0.000000
		currentRate = 0.0
	EndIf

	; just to be sane
	If fPeriod < 0.000000
		fPeriod = 0.0
	EndIf

	return( ( currentRate * fPeriod ) / 100.0 )
endFunction

float Function getSleepDeltaPeriod( float startTime, float endTime )
	return( (endTime - startTime) * 86400.0 )
endFunction

bool Function isSleepEnabledFor( string avName )
	if ( RRControl.isRRAVEnabled( avName ) )
		int fType = RRControl.getFunctionType( "Sleep", avName )

		if ( fType == iVANILLA )
			return false
		else
			return true
		endIf
	else
		return false
	endIf
endFunction


;
; EVENTS & STATES ---------------------------------------------------------------------------------
;

Event OnSleepStart( float afSleepStartTime, float afDesiredSleepEndTime )
	; Mark that we slept
	;
	_isWasSleep = true

	; Just remember current values, because sleep may be interrupted
	; at any time.
	; After awake we recalculate values based on own formulas!
	;
	_sleepStartTime = afSleepStartTime
	_sleepEndTime = afDesiredSleepEndTime
EndEvent

Event OnSleepStop( bool abInterrupted )
	_isInterrupted = abInterrupted
EndEvent

Event OnMenuOpen( String MenuName )
	if ( MenuName != "Sleep/Wait Menu" )
		return
	endIf

	int avIndex = 0
	while ( avIndex < avNamesList.Length )
		string avName = avNamesList[avIndex]

		beforeSleepValuesList[avIndex] = PlayerREF.GetActorValue( avName )
		afterSleepValuesList[avIndex] = beforeSleepValuesList[avIndex]

		avIndex += 1
	endWhile
EndEvent

Event OnMenuClose( String MenuName )
	if ( MenuName != "Sleep/Wait Menu" )
		return
	endIf

	if ( !_isWasSleep )
		return
	endIf

	float sleepPeriod = 0.0

	if ( _isInterrupted )
		float endTime = Utility.GetCurrentGameTime()
		sleepPeriod = getSleepDeltaPeriod( _sleepStartTime, endTime )
	else
		sleepPeriod = getSleepDeltaPeriod( _sleepStartTime, _sleepEndTime )
	endif

	bool bEnable = false
	int avIndex = 0
	while ( avIndex < avNamesList.Length )
		if ( isSleepEnabledFor( avNamesList[avIndex] ) )
			setAVAfterSleep( avIndex, sleepPeriod )

			if ( !bEnable )
				bEnable = true
			endIf
		endif

		avIndex += 1
	endWhile

	if ( bEnable )
		if ( RRControl.isDebug() )
			Debug.Notification( "Sleep period: " + sleepPeriod )
		endif

		GotoState( "OnSleepCalculation" )
		RegisterForSingleUpdate( RRUpdateSleepAt.GetValue() )
	endIf
EndEvent

State OnSleepCalculation
	event OnUpdate()
		if ( !_isWasSleep )
			UnregisterForUpdate()
			GotoState( "Inactive" )
			return
		EndIF

		int avIndex = 0
		while ( avIndex < avNamesList.Length )
			string avName = avNamesList[avIndex]

			if ( isSleepEnabledFor( avName ) )
				float newValue = afterSleepValuesList[avIndex]
				float deltaAV = PlayerREF.GetActorValue( avName ) - newValue

				if ( RRControl.isDebug() )
					Debug.Notification( "Set [" + avName + "]: " + newValue )
				endif

				if ( deltaAV > 0.000000 )
					PlayerREF.DamageAV( avName, deltaAV )
				else
					PlayerREF.RestoreAV( avName, deltaAV )
				endif
			endif

			avIndex += 1
		endWhile

		_isWasSleep = false

		UnregisterForUpdate()
		GotoState( "Inactive" )
	endEvent
endState

State Inactive
	Event OnUpdate()
		; nop
	EndEvent
EndState

Event OnUpdate()
	; nop
EndEvent
