Scriptname RRWhenWaitScript extends Quest  
{Recalculate Health, Magicka and Stamina after a wait}

;
; GLOBAL VARIABLES --------------------------------------------------------------------------------
;

; Scripts
RRControlScript Property RRControl Auto

; Game
Actor Property PlayerREF Auto

; Timers
GlobalVariable Property RRUpdateWaitAt Auto

; Tweaks
GlobalVariable Property RRMinWaitPeriod Auto

; Constants
GlobalVariable Property RRFunctionNoHeal Auto
GlobalVariable Property RRFunctionDefault Auto
GlobalVariable Property RRFunctionUserDefined Auto
GlobalVariable Property RRFunctionVanilla Auto


;
; PRIVATE VARIABLES -------------------------------------------------------------------------------
;

float            _waitStartTime
bool             _sleepMode

string[]         avNamesList
float[]          beforeWaitValuesList
float[]          afterWaitValuesList

int              iNO_HEAL
int              iDEFAULT
int              iDEFINED
int              iVANILLA


;
; FUNCTIONS ---------------------------------------------------------------------------------------
;

Function initializeAll()
	avNamesList = new string[3]
	avNamesList[0] = "Health"
	avNamesList[1] = "Magicka"
	avNamesList[2] = "Stamina"

	beforeWaitValuesList = new float[3]
	beforeWaitValuesList[0] = 0.0
	beforeWaitValuesList[1] = 0.0
	beforeWaitValuesList[2] = 0.0

	afterWaitValuesList = new float[3]
	afterWaitValuesList[0] = 0.0
	afterWaitValuesList[1] = 0.0
	afterWaitValuesList[2] = 0.0

	_sleepMode = false

	iNO_HEAL = RRFunctionNoHeal.GetValueInt()
	iDEFAULT = RRFunctionDefault.GetValueInt()
	iDEFINED = RRFunctionUserDefined.GetValueInt()
	iVANILLA = RRFunctionVanilla.GetValueInt()

	if ( iDEFAULT == 0 )
		Debug.Notification( "RR (wait): incorrect installation!" )
	endIf
endFunction

Function reset()
	deactivate()

	if ( !RRControl.isActive() )
		return
	endIf

	bool bEnable = false

	int iIndex = 0
	while ( iIndex < avNamesList.Length )
		if ( !bEnable )
			if ( isWaitEnabledFor( avNamesList[iIndex] ) )
				bEnable = true
			endIf
		endIf

		iIndex += 1
	endWhile

	if ( bEnable )
		RegisterForMenu("Sleep/Wait Menu")
		RegisterForSleep()
	endIf
endFunction

Function deactivate()
	UnregisterForMenu("Sleep/Wait Menu")
	UnregisterForSleep()
endFunction

Function setAVAfterWait( int avIndex, float period )
	string avName = avNamesList[avIndex]
	int fType = RRControl.getFunctionType( "Wait", avName )
	float rv = 0.0

	if ( fType == iNO_HEAL )
		; nop
	elseif ( fType == iDEFAULT )
		rv = getModificator( period, RRControl.getRateValue( avName ) )
	elseif ( fType == iDEFINED )
		rv = getModificator( period, RRControl.getRateByType( "Wait", avName ) )
	elseif ( fType == iVANILLA )
		rv = getModificator( period, RRControl.getCurrentRateByIndex( avIndex ) )
	else
		return
	endIf

	if ( RRControl.isDebug() )
		Debug.Notification( "Gained after a wait [" + avName + "]: " + rv )
	endIf

	afterWaitValuesList[avIndex] = beforeWaitValuesList[avIndex] + rv
endFunction

; Count how much HP we able to restore
; when currentRate == 0 we dont regenerate at all
;
float Function getModificator( float fPeriod, float currentRate )
	; Rate is always positive number: our functions returns -1.0 if smth goes wrong
	If currentRate < 0.000000
		currentRate = 0.0
	EndIf

	; just to be sane
	If fPeriod < 0.000000
		fPeriod = 0.0
	EndIf

	return( ( currentRate * fPeriod ) / 100.0 )
endFunction

float Function getWaitDeltaPeriod( float startTime, float endTime )
	return( (endTime - startTime) * 86400.0 )
endFunction

bool Function isWaitEnabledFor( string avName )
	if ( RRControl.isRRAVEnabled( avName ) )
		int fType = RRControl.getFunctionType( "Wait", avName )

		if ( fType == iVANILLA )
			return false
		else
			return true
		endIf
	else
		return false
	endIf
endFunction


;
; EVENTS & STATES ---------------------------------------------------------------------------------
;

event OnSleepStart( float afSleepStartTime, float afDesiredSleepEndTime )
	_sleepMode = true
endEvent

event OnSleepStop( bool abInterrupted )
	; nop
endEvent

event OnMenuOpen( String MenuName )
	if ( MenuName != "Sleep/Wait Menu" )
		return
	endIf

	; Just remember current values, because wait may be interrupted
	; at any time.
	; After awake we recalculate values based on own formulas!
	;
	_waitStartTime = Utility.GetCurrentGameTime()

	int avIndex = 0
	while ( avIndex < avNamesList.Length )
		beforeWaitValuesList[avIndex] = PlayerREF.GetActorValue( avNamesList[avIndex] )
		afterWaitValuesList[avIndex] = beforeWaitValuesList[avIndex]

		avIndex += 1
	endWhile
endEvent

event OnMenuClose( String MenuName )
	if ( MenuName != "Sleep/Wait Menu" )
		return
	endIf

	float waitPeriod = getWaitDeltaPeriod( _waitStartTime, Utility.GetCurrentGameTime() )

	if ( _sleepMode )
		; such a fool :-)
		;
		_sleepMode = false
		return
	endIf

	bool bEnable = false
	int avIndex = 0
	while ( avIndex < avNamesList.Length )
		if ( isWaitEnabledFor( avNamesList[avIndex] ) )
			setAVAfterWait( avIndex, waitPeriod )

			if ( !bEnable )
				bEnable = true
			endIf
		endif

		avIndex += 1
	endWhile

	if ( bEnable )
		if ( RRControl.isDebug() )
			Debug.Notification( "Sleep/Wait period: " + waitPeriod )
		endif

		if ( waitPeriod > RRMinWaitPeriod.GetValue() )
			GotoState( "OnWaitCalculation" )
			RegisterForSingleUpdate( RRUpdateWaitAt.GetValue() )
		endIf
	endIf
endEvent

State OnWaitCalculation
	event OnUpdate()
		if ( _sleepMode )
			; such a fool; better check this twice :-D
			;
			_sleepMode = false
			UnregisterForUpdate()
			GotoState( "Inactive" )
			return
		endIf

		int avIndex = 0
		while ( avIndex < avNamesList.Length )
			string avName = avNamesList[avIndex]

			if ( isWaitEnabledFor( avName ) )
				float newValue = afterWaitValuesList[avIndex]
				float deltaAV = PlayerREF.GetActorValue( avName ) - newValue

				if ( RRControl.isDebug() )
					Debug.Notification( "Set [" + avName + "] (wait): " + newValue )
				endif

				if ( deltaAV > 0.000000 )
					PlayerREF.DamageAV( avName, deltaAV )
				else
					PlayerREF.RestoreAV( avName, deltaAV )
				endif
			endif

			avIndex += 1
		endWhile

		UnregisterForUpdate()
		GotoState( "Inactive" )
	endEvent
endState

State Inactive
	Event OnUpdate()
		; nop
	EndEvent
EndState

Event OnUpdate()
	; nop
EndEvent
